package Padre::Wx::FBP::SessionManager;

## no critic

# This module was generated by Padre::Plugin::FormBuilder::Perl.
# To change this module edit the original .fbp file and regenerate.
# DO NOT MODIFY THIS FILE BY HAND!

use 5.008005;
use utf8;
use strict;
use warnings;
use Padre::Wx ();
use Padre::Wx::Role::Main ();

our $VERSION = '0.98';
our @ISA     = qw{
	Padre::Wx::Role::Main
	Wx::Dialog
};

sub new {
	my $class  = shift;
	my $parent = shift;

	my $self = $class->SUPER::new(
		$parent,
		-1,
		Wx::gettext("Session Manager"),
		Wx::DefaultPosition,
		[ 480, 300 ],
		Wx::DEFAULT_DIALOG_STYLE,
	);

	$self->{list} = Wx::ListCtrl->new(
		$self,
		-1,
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::LC_REPORT | Wx::LC_SINGLE_SEL,
	);

	Wx::Event::EVT_LIST_COL_CLICK(
		$self,
		$self->{list},
		sub {
			shift->list_col_click(@_);
		},
	);

	Wx::Event::EVT_LIST_ITEM_ACTIVATED(
		$self,
		$self->{list},
		sub {
			shift->list_item_activated(@_);
		},
	);

	Wx::Event::EVT_LIST_ITEM_SELECTED(
		$self,
		$self->{list},
		sub {
			shift->list_item_selected(@_);
		},
	);

	$self->{autosave} = Wx::CheckBox->new(
		$self,
		-1,
		Wx::gettext("Save session automatically"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);

	$self->{m_staticline46} = Wx::StaticLine->new(
		$self,
		-1,
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::LI_HORIZONTAL,
	);

	$self->{open} = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Open Session"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);
	$self->{open}->SetDefault;

	Wx::Event::EVT_BUTTON(
		$self,
		$self->{open},
		sub {
			shift->open_clicked(@_);
		},
	);

	$self->{delete} = Wx::Button->new(
		$self,
		-1,
		Wx::gettext("Delete Session"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);

	Wx::Event::EVT_BUTTON(
		$self,
		$self->{delete},
		sub {
			shift->delete_clicked(@_);
		},
	);

	$self->{cancel} = Wx::Button->new(
		$self,
		Wx::ID_CANCEL,
		Wx::gettext("Close"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);

	my $bSizer120 = Wx::BoxSizer->new(Wx::HORIZONTAL);
	$bSizer120->Add( $self->{open}, 0, Wx::ALL, 5 );
	$bSizer120->Add( $self->{delete}, 0, Wx::ALL, 5 );
	$bSizer120->Add( 0, 0, 1, Wx::EXPAND, 5 );
	$bSizer120->Add( $self->{cancel}, 0, Wx::ALL, 5 );

	my $bSizer119 = Wx::BoxSizer->new(Wx::VERTICAL);
	$bSizer119->Add( $self->{list}, 1, Wx::ALL | Wx::EXPAND, 5 );
	$bSizer119->Add( $self->{autosave}, 0, Wx::ALL | Wx::EXPAND, 5 );
	$bSizer119->Add( $self->{m_staticline46}, 0, Wx::EXPAND | Wx::LEFT | Wx::RIGHT | Wx::TOP, 5 );
	$bSizer119->Add( $bSizer120, 0, Wx::EXPAND, 5 );

	$self->SetSizer($bSizer119);
	$self->Layout;

	return $self;
}

sub list_col_click {
	$_[0]->main->error('Handler method list_col_click for event list.OnListColClick not implemented');
}

sub list_item_activated {
	$_[0]->main->error('Handler method list_item_activated for event list.OnListItemActivated not implemented');
}

sub list_item_selected {
	$_[0]->main->error('Handler method list_item_selected for event list.OnListItemSelected not implemented');
}

sub open_clicked {
	$_[0]->main->error('Handler method open_clicked for event open.OnButtonClick not implemented');
}

sub delete_clicked {
	$_[0]->main->error('Handler method delete_clicked for event delete.OnButtonClick not implemented');
}

1;

# Copyright 2008-2013 The Padre development team as listed in Padre.pm.
# LICENSE
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl 5 itself.
