package Padre::Project::Perl::MI;

# Perl project driven by Module::Install

use 5.008005;
use strict;
use warnings;
use Padre::Project::Perl::EUMM ();

our $VERSION = '0.98';
our @ISA     = 'Padre::Project::Perl::EUMM';

1;


__END__

# Copyright 2008-2013 The Padre development team as listed in Padre.pm.
# LICENSE
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl 5 itself.

