package Padre::Wx::FBP::SLOC;

## no critic

# This module was generated by Padre::Plugin::FormBuilder::Perl.
# To change this module edit the original .fbp file and regenerate.
# DO NOT MODIFY THIS FILE BY HAND!

use 5.008005;
use utf8;
use strict;
use warnings;
use Padre::Wx ();
use Padre::Wx::Role::Main ();

our $VERSION = '0.98';
our @ISA     = qw{
	Padre::Wx::Role::Main
	Wx::Dialog
};

sub new {
	my $class  = shift;
	my $parent = shift;

	my $self = $class->SUPER::new(
		$parent,
		-1,
		Wx::gettext("Project Statistics"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::DEFAULT_DIALOG_STYLE,
	);

	$self->{root} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);
	$self->{root}->SetFont(
		Wx::Font->new( Wx::NORMAL_FONT->GetPointSize, 70, 90, 92, 0, "" )
	);

	my $m_staticline48 = Wx::StaticLine->new(
		$self,
		-1,
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::LI_HORIZONTAL,
	);

	my $m_staticText210 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Files:"),
	);

	$self->{files} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText213 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Code Lines:"),
	);

	$self->{code} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText215 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Comments Lines:"),
	);

	$self->{comment} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText217 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Blank Lines:"),
	);

	$self->{blank} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText218 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Constructive Cost Model (COCOMO)"),
	);
	$m_staticText218->SetFont(
		Wx::Font->new( Wx::NORMAL_FONT->GetPointSize, 70, 90, 92, 0, "" )
	);

	my $m_staticline50 = Wx::StaticLine->new(
		$self,
		-1,
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::LI_HORIZONTAL,
	);

	my $m_staticText219 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Mythical Man Months:"),
	);

	$self->{pax_months} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText221 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Estimated Project Years:"),
	);

	$self->{cal_years} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText223 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Number of Developers:"),
	);

	$self->{dev_count} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticText225 = Wx::StaticText->new(
		$self,
		-1,
		Wx::gettext("Development Cost (USD):"),
	);

	$self->{dev_cost} = Wx::StaticText->new(
		$self,
		-1,
		'',
	);

	my $m_staticline49 = Wx::StaticLine->new(
		$self,
		-1,
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::LI_HORIZONTAL,
	);

	$self->{cancel} = Wx::Button->new(
		$self,
		Wx::ID_CANCEL,
		Wx::gettext("Close"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);
	$self->{cancel}->SetDefault;

	my $fgSizer29 = Wx::FlexGridSizer->new( 4, 2, 10, 20 );
	$fgSizer29->AddGrowableCol(1);
	$fgSizer29->SetFlexibleDirection(Wx::HORIZONTAL);
	$fgSizer29->SetNonFlexibleGrowMode(Wx::FLEX_GROWMODE_SPECIFIED);
	$fgSizer29->Add( $m_staticText210, 0, Wx::EXPAND, 5 );
	$fgSizer29->Add( $self->{files}, 0, Wx::ALIGN_RIGHT, 5 );
	$fgSizer29->Add( $m_staticText213, 0, Wx::EXPAND, 5 );
	$fgSizer29->Add( $self->{code}, 0, Wx::ALIGN_RIGHT, 5 );
	$fgSizer29->Add( $m_staticText215, 0, Wx::EXPAND, 5 );
	$fgSizer29->Add( $self->{comment}, 0, Wx::ALIGN_RIGHT, 5 );
	$fgSizer29->Add( $m_staticText217, 0, Wx::EXPAND, 5 );
	$fgSizer29->Add( $self->{blank}, 0, Wx::ALIGN_RIGHT, 5 );

	my $fgSizer30 = Wx::FlexGridSizer->new( 4, 2, 10, 20 );
	$fgSizer30->AddGrowableCol(1);
	$fgSizer30->SetFlexibleDirection(Wx::HORIZONTAL);
	$fgSizer30->SetNonFlexibleGrowMode(Wx::FLEX_GROWMODE_SPECIFIED);
	$fgSizer30->Add( $m_staticText219, 0, Wx::EXPAND, 5 );
	$fgSizer30->Add( $self->{pax_months}, 0, Wx::ALIGN_RIGHT, 5 );
	$fgSizer30->Add( $m_staticText221, 0, Wx::EXPAND, 5 );
	$fgSizer30->Add( $self->{cal_years}, 0, Wx::ALIGN_RIGHT, 5 );
	$fgSizer30->Add( $m_staticText223, 0, Wx::EXPAND, 5 );
	$fgSizer30->Add( $self->{dev_count}, 0, Wx::ALIGN_RIGHT, 5 );
	$fgSizer30->Add( $m_staticText225, 0, Wx::EXPAND, 5 );
	$fgSizer30->Add( $self->{dev_cost}, 0, Wx::ALIGN_RIGHT, 5 );

	my $bSizer124 = Wx::BoxSizer->new(Wx::HORIZONTAL);
	$bSizer124->SetMinSize( [ 150, -1 ] );
	$bSizer124->Add( 0, 0, 1, Wx::EXPAND, 5 );
	$bSizer124->Add( $self->{cancel}, 0, Wx::ALL, 5 );

	my $bSizer123 = Wx::BoxSizer->new(Wx::VERTICAL);
	$bSizer123->Add( $self->{root}, 0, Wx::ALL | Wx::EXPAND, 5 );
	$bSizer123->Add( $m_staticline48, 0, Wx::EXPAND, 5 );
	$bSizer123->Add( $fgSizer29, 0, Wx::ALL | Wx::EXPAND, 5 );
	$bSizer123->Add( 0, 10, 0, Wx::EXPAND, 5 );
	$bSizer123->Add( $m_staticText218, 0, Wx::ALL, 5 );
	$bSizer123->Add( $m_staticline50, 0, Wx::BOTTOM | Wx::EXPAND, 5 );
	$bSizer123->Add( $fgSizer30, 1, Wx::ALL | Wx::EXPAND, 5 );
	$bSizer123->Add( $m_staticline49, 0, Wx::EXPAND, 5 );
	$bSizer123->Add( $bSizer124, 0, Wx::EXPAND, 5 );

	$self->SetSizerAndFit($bSizer123);
	$self->Layout;

	return $self;
}

1;

# Copyright 2008-2013 The Padre development team as listed in Padre.pm.
# LICENSE
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl 5 itself.

